"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getNodeDir = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const fetch_1 = __importStar(require("@pnpm/fetch"));
const package_store_1 = require("@pnpm/package-store");
const store_path_1 = __importDefault(require("@pnpm/store-path"));
const tarball_fetcher_1 = __importStar(require("@pnpm/tarball-fetcher"));
const adm_zip_1 = __importDefault(require("adm-zip"));
const rename_overwrite_1 = __importDefault(require("rename-overwrite"));
const tempy_1 = __importDefault(require("tempy"));
const load_json_file_1 = __importDefault(require("load-json-file"));
const write_json_file_1 = __importDefault(require("write-json-file"));
async function getNodeDir(opts) {
    var _a, _b;
    const nodesDir = path_1.default.join(opts.pnpmHomeDir, 'nodes');
    let wantedNodeVersion = (_a = opts.useNodeVersion) !== null && _a !== void 0 ? _a : (_b = (await readNodeVersionsManifest(nodesDir))) === null || _b === void 0 ? void 0 : _b.default;
    await fs_1.default.promises.mkdir(nodesDir, { recursive: true });
    fs_1.default.writeFileSync(path_1.default.join(nodesDir, 'pnpm-workspace.yaml'), '', 'utf8');
    if (wantedNodeVersion == null) {
        const response = await fetch_1.default('https://registry.npmjs.org/node');
        wantedNodeVersion = (await response.json())['dist-tags'].lts;
        if (wantedNodeVersion == null) {
            throw new Error('Could not resolve LTS version of Node.js');
        }
        await write_json_file_1.default(path_1.default.join(nodesDir, 'versions.json'), {
            default: wantedNodeVersion,
        });
    }
    const versionDir = path_1.default.join(nodesDir, wantedNodeVersion);
    if (!fs_1.default.existsSync(versionDir)) {
        await installNode(wantedNodeVersion, versionDir, opts);
    }
    return process.platform === 'win32' ? versionDir : path_1.default.join(versionDir, 'bin');
}
exports.getNodeDir = getNodeDir;
async function installNode(wantedNodeVersion, versionDir, opts) {
    await fs_1.default.promises.mkdir(versionDir, { recursive: true });
    const { tarball, pkgName } = getNodeJSTarball(wantedNodeVersion);
    const fetchFromRegistry = fetch_1.createFetchFromRegistry(opts);
    if (tarball.endsWith('.zip')) {
        await downloadAndUnpackZip(fetchFromRegistry, tarball, versionDir, pkgName);
        return;
    }
    const getCredentials = () => ({ authHeaderValue: undefined, alwaysAuth: undefined });
    const fetch = tarball_fetcher_1.default(fetchFromRegistry, getCredentials, {
        retry: {
            maxTimeout: opts.fetchRetryMaxtimeout,
            minTimeout: opts.fetchRetryMintimeout,
            retries: opts.fetchRetries,
            factor: opts.fetchRetryFactor,
        },
        timeout: opts.fetchTimeout,
    });
    const storeDir = await store_path_1.default(process.cwd(), opts.storeDir);
    const cafsDir = path_1.default.join(storeDir, 'files');
    const cafs = package_store_1.createCafsStore(cafsDir);
    const { filesIndex } = await fetch.tarball(cafs, { tarball }, {
        lockfileDir: process.cwd(),
    });
    await cafs.importPackage(versionDir, {
        filesResponse: {
            filesIndex: await tarball_fetcher_1.waitForFilesIndex(filesIndex),
            fromStore: false,
        },
        force: true,
    });
}
async function downloadAndUnpackZip(fetchFromRegistry, zipUrl, targetDir, pkgName) {
    const response = await fetchFromRegistry(zipUrl);
    const tmp = path_1.default.join(tempy_1.default.directory(), 'pnpm.zip');
    const dest = fs_1.default.createWriteStream(tmp);
    await new Promise((resolve, reject) => {
        response.body.pipe(dest).on('error', reject).on('close', resolve);
    });
    const zip = new adm_zip_1.default(tmp);
    const nodeDir = path_1.default.dirname(targetDir);
    zip.extractAllTo(nodeDir, true);
    await rename_overwrite_1.default(path_1.default.join(nodeDir, pkgName), targetDir);
    await fs_1.default.promises.unlink(tmp);
}
function getNodeJSTarball(nodeVersion) {
    const platform = process.platform === 'win32' ? 'win' : process.platform;
    const arch = platform === 'win' && process.arch === 'ia32' ? 'x86' : process.arch;
    const extension = platform === 'win' ? 'zip' : 'tar.gz';
    const pkgName = `node-v${nodeVersion}-${platform}-${arch}`;
    return {
        pkgName,
        tarball: `https://nodejs.org/download/release/v${nodeVersion}/${pkgName}.${extension}`,
    };
}
async function readNodeVersionsManifest(nodesDir) {
    try {
        return await load_json_file_1.default(path_1.default.join(nodesDir, 'versions.json'));
    }
    catch (err) {
        if (err.code === 'ENOENT') {
            return {};
        }
        throw err;
    }
}
//# sourceMappingURL=node.js.map